@@ -11,8 +11,9 @@
 #' containing a list of multivariate functional principal component analysis
 #' models estimated
 #' on functional data each evolving up to an intermediate domain point.
-#' @param components
-#' See \code{\link{pca_mfd}}.
+#' @param components_list
+#' A list of components given as input to \code{\link{pca_mfd}}
+#' for each intermediate domain point.
 #' @param mfdobj_x_test
 #' A list created using
 #' \code{\link{get_mfd_df_real_time}} or
@@ -71,14 +72,18 @@
 #'                                          lambda = 1e-2,
 #'                                          k_seq = c(0.5, 1))
 #' pca_list <- pca_mfd_real_time(mfdobj_x1_list)
+#' components_list <- lapply(pca_list, function(x) {
+#'   sum(cumsum(x$varprop) <= .8)
+#' })
+#'
 #' cclist <- control_charts_pca_mfd_real_time(
 #'   pca_list = pca_list,
-#'   components = 1:3,
+#'   components_list = 1:3,
 #'   mfdobj_x_test = mfdobj_x2_list)
 #' plot_control_charts_real_time(cclist, 1)
 #'
 control_charts_pca_mfd_real_time <- function(pca_list,
-                                             components,
+                                             components_list,
                                              mfdobj_x_test,
                                              mfdobj_x_tuning = NULL,
                                              alpha = list(T2 = .0125,
@@ -98,7 +103,7 @@ control_charts_pca_mfd_real_time <- function(pca_list,
 
     cclist_kk <- control_charts_pca(
       pca = pca_list[[ii]],
-      components = components,
+      components = components_list[[ii]],
       tuning_data = mfdobj_x_tuning[[ii]],
       newdata = mfdobj_x_test[[ii]],
       alpha = alpha,
@@ -128,7 +133,7 @@ control_charts_pca_mfd_real_time <- function(pca_list,
       cl <- makeCluster(ncores)
       clusterExport(cl,
                     c("pca_list",
-                      "components",
+                      "components_list",
                       "mfdobj_x_test",
                       "mfdobj_x_tuning",
                       "alpha",
